package com.huiju.welcome.common;

import com.huiju.welcome.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * @author FXF
 * @date 2019-03-14
 */
@ServerEndpoint("/ws/{type}/{sid}")
@Component
public class WebSocketServer {
	private static Logger log = LoggerFactory.getLogger(WebSocketServer.class);
	
	/**
	 * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
	 */
	private static int onlineCount = 0;
	/**
	 * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
	 */
	private static CopyOnWriteArraySet<WebSocketServer> webSocketSet = new CopyOnWriteArraySet<WebSocketServer>();
	
	/**
	 * 与某个客户端的连接会话，需要通过它来给客户端发送数据
	 */
	private Session session;

	/**
	 * 接收sid
	 */
	private String sid="";
	String type = "";

	/**
	 * 连接建立成功调用的方法*/
	@OnOpen
	public void onOpen(Session session,@PathParam("sid") String sid, @PathParam("type") String type) {
		if (null == sid || "".equals(sid)) return;

		this.sid = sid;
		this.type = type;
		this.session = session;
		//加入set中
		webSocketSet.add(this);
		//在线数加1
		addOnlineCount();

		log.info("有新窗口 [{} - {}] 开始监听, 当前在线人数为 {}", type, sid, getOnlineCount());
	}
	
	/**
	 * 连接关闭调用的方法
	 */
	@OnClose
	public void onClose() {
		webSocketSet.remove(this);
		subOnlineCount();
		log.info("有一连接关闭！当前在线人数为" + getOnlineCount());
	}
	
	/**
	 * 收到客户端消息后调用的方法
	 *
	 * @param message 客户端发送过来的消息*/
	@OnMessage
	public void onMessage(String message, Session session) {
		log.info("收到来自窗口 [{} - {}] 的信息: {}", this.type, this.sid,  message);

		if (null == message || "".equals(message)) {
			return;
		}

		if ("ping".equals(message)) {
			try {
				sendMessage("pong");
			} catch (IOException e) {}
		}

		// 目前需求, 没有需要通过 ws 来处理的业务
	}
	
	/**
	 *
	 * @param session
	 * @param error
	 */
	@OnError
	public void onError(Session session, Throwable error) {
		log.error("发生错误: " + error.getMessage());
		error.printStackTrace();
	}

	/**
	 * 实现服务器主动推送
	 */
	public void sendMessage(String message) throws IOException {
		log.info("发送消息 [{}-{}]: {}", type, sid, message);
		this.session.getBasicRemote().sendText(message);
	}
	
	
	/**
	 * 群发自定义消息
	 * */
	public static void sendInfo(String message, String type, String sid) throws Exception {
		if (StringUtils.isEmpty(type) || StringUtils.isEmpty(message)) {
			throw new Exception("发送数据非法, 部分字段为空");
		}

		for (WebSocketServer item : webSocketSet) {
			try {
				if (StringUtils.isEmpty(sid) && "miniapp".equals(type) && item.type.equals(type)) {
					// 小程序消息, 如果 sid 为空, 则广播
					item.sendMessage(message);
				} else  if(item.sid.equals(sid) && item.type.equals(type)){
					item.sendMessage(message);
					return;
				}
			} catch (IOException e) {
				log.error("发送数据失败: [{}-{}], 发送内容: {}", type, sid, message);
				log.error("错误内容: {}", e.getMessage());
				continue;
			}
		}
	}
	
	public static synchronized int getOnlineCount() {
		return onlineCount;
	}
	
	public static synchronized void addOnlineCount() {
		WebSocketServer.onlineCount++;
	}
	
	public static synchronized void subOnlineCount() {
		WebSocketServer.onlineCount--;
	}
}
